require_relative "../lib/meta_parse"

module MetaParseExamples
  class ParseNumber
    include MetaParse

    ##
    # Using Regexps and built-in parsers will always be faster.  (Optimized by implementation.)
    # See :pos_integer2 for reference.
    
    match_method :integer do
      seq(/-?[0-9]+/) { |(digits)| digits.to_i }
    end

    ##
    # Match any number with regexp, then select appropriate built-in parser to produce number.
    
    match_method :number do
      seq(/-?[0-9]+(\.[0-9]+)?([eE]-?[0-9])?/) { |(v)| (v =~ /\./) ? v.to_f : v.to_i }
    end
    
    match_method :sign do
      seq [:'?', '-'] { |(m)| (m.count > 0) ? -1 : 1 }
    end
        

    ##
    # An example of how to parse an integer using a repetition reducer.
    # For the sake of simpicity, only accept positive integers.
    
    match_method :pos_integer2 do
      rep /[0-9]/, nil, nil, lambda { |a, d| (a * 10 ) + (d.ord - '0'.ord) }, 0
    end

    ##
    # Exmample of how to specify source and compile explicitly.
    
    match_method :pos_integer3 do
      self.comp [:*, /[0-9]/, nil, nil, lambda { |a, d| (a * 10 ) + (d.ord - '0'.ord) }, 0]
    end


    ##
    # Convert an integer into decimal (assuming leading decimal point).
    
    def self.decimalize(integer)
      (integer.to_f) / 10**(Math::log10(integer).floor + 1)
    end

    ##
    # Match and return a decimal number starting with the decimal point.
    
    match_method :decimal do
      seq( '.', :integer) { |dot, integer| decimalize(integer) }
    end

    ##
    # Match and return a positive or negative number, with or without a decimal part.
    
    match_method :number2 do
      seq( :sign, :pos_integer2, [:'?', :decimal] ) { |(sign, integer, (decimal))|
        sign * (integer + (decimal || 0)) }
    end
  end  
end
